#!/usr/bin/perl
## ######### PROJECT NAME : ##########
## 
## disk_object.pm for aspiq-core
## 
## ######### PROJECT DESCRIPTION : ###
## 
## this file brings disk and partition support for aspiq core.
## 
## ###################################
## 
## Made by Boutonnet Alexandre
## Login   <aboutonnet@intellique.com>
## 
## Started on  Thu Feb  7 15:19:01 2008 Boutonnet Alexandre
## Last update Fri Nov  7 11:31:35 2008 Boutonnet Alexandre
## 
## ###################################
##

package disk;

use strict;
use warnings;

use Data::Dumper;

use Lib_Disk;

# Error tab
my $error_hash = {
    0 => 'Success',
    100 => 'Internal error',
    101 => 'Device parameter not found',
    102 => 'Device name not found',
    103 => 'Partition name not found',
    104 => 'Unable to get informations for : ',
    105 => 'No start value found for partition : ',
    106 => 'No end value found for partition : ',
    107 => 'Set ',
    108 => 'Delete ',
    200 => 'Read only mode : this command can not be executed',
    201 => 'Partition does not belong to the disk',
    300 => 'This method does nothing in this plugin',
};


# This function is the constructor of this object
# The parameter of this method is an instance of the
# log object.
sub new
{
    my $log_obj = shift; # Getting the log object.

    my $OBJ = {}; # This had to contain object's personnal datas.

    $OBJ->{log} = $log_obj;

    bless($OBJ);

    return($OBJ);
}

# This function permit to user to initiate this object
# The parameter of this method is the read-only boolean
sub init
{
    my ($OBJ, $read_only) = @_;

    $OBJ->{read_only} = $read_only;

    return(0);
}

# This method permit to user to get info
# The parameter is an hash containing parameters.
sub get
{
    my ($OBJ, $param_hash) = @_;

    # Testing if hash contains device key
    return(_make_error_hash(101, $error_hash->{101})) if (!exists($param_hash->{'device'}));

    my $hash_return = {
	'device' => {},
    };
    
    foreach my $device (@{$param_hash->{'device'}})
    {
	# Testing if hash contains name key
	return(_make_error_hash(102, $error_hash->{102})) if (!exists($device->{'name'}));

	# This variable contains the device name
	my $device_name = $device->{'name'}[0];

	# If there is some partitions
	if (exists($device->{'partition'}))
	{
	    foreach my $partition (@{$device->{'partition'}})
	    {
		# Testing if partitions name are present
		return(_make_error_hash(103, $error_hash->{103})) if (!exists($partition->{'name'}));
		
		# Testing if partition belongs to the device
		return(_make_error_hash(201, $error_hash->{201})) if ($partition->{'name'}[0] !~ m/^($device_name).*/);
		
		my $hash_info = Lib_Disk::get_partition_info($partition->{'name'}[0]);
		return(_make_error_hash(104, $error_hash->{104}.$partition->{'name'}[0])) if (!$hash_info);
		
		$hash_return->{'device'}->{$device_name}->{'partition'}->{$partition->{'name'}[0]} = 
		    $hash_info->{$partition->{'name'}[0]};
	    }
	}
	else
	{
	    my $hash_info = Lib_Disk::get_one_disk_info($device_name);
	    return(_make_error_hash(104, $error_hash->{104}.$device_name)) if (!$hash_info);
	    
	    $hash_return->{'device'}->{$device_name} = $hash_info->{$device_name};
	}
    }

    return (_make_recursive_hash($hash_return));
}

# This method permit to user to make an action
# The parameter is an hash containing parameters.
sub set
{
    my ($OBJ, $param_hash) = @_;

    # Testing the read only mode..
    return(_make_error_hash(200, $error_hash->{200})) if ($OBJ->{read_only});

    # Testing if hash contains device key
    return(_make_error_hash(101, $error_hash->{101})) if (!exists($param_hash->{'device'}));

    # Making tests of the incomming hash
    foreach my $device (@{$param_hash->{'device'}})
    {
	# Testing if hash contains name key
	return(_make_error_hash(102, $error_hash->{102})) if (!exists($device->{'name'}));

	# This variable contains the device name
	my $device_name = $device->{'name'}[0];

	# If there is some partitions
	if (exists($device->{'partition'}))
	{
	    foreach my $partition (@{$device->{'partition'}})
	    {
		# Testing if partitions name are present
		return(_make_error_hash(103, $error_hash->{103})) if (!exists($partition->{'name'}));

		# Testing if partition belongs to the device
		return(_make_error_hash(201, $error_hash->{201})) if ($partition->{'name'}[0] !~ m/^($device_name).*/);
		
		# Testing for start value only if start exists and is not empty
		return(_make_error_hash(105, $error_hash->{105}.$partition->{'name'}[0]))
		    if (exists($partition->{'start'}) && !$partition->{'start'}[0]);

		# Testing for end value
		return(_make_error_hash(106, $error_hash->{106}.$partition->{'name'}[0]))
		    if (exists($partition->{'start'}) &&
			(!exists($partition->{'end'}) || !$partition->{'end'}[0]));
	    }
	}
    }

    # Preparing hash to be readable by Lib_disk
    my $lib_hash = _unmake_hash($param_hash);

    my ($err, $msg) = Lib_Disk::part_disk($lib_hash);
    # Testing for error..
    return(_make_error_hash(107, $error_hash->{107}.$msg)) if ($err);

    return(_make_error_hash(0, $error_hash->{0}));
}

# This method permit to the user to get all info
# There is no parameter to this method.
sub info
{
    my $disk_hash = Lib_Disk::get_disk_info();

    return(_make_error_hash(100, $error_hash->{100})) if (!$disk_hash);


    my $tmp_hash = {
	'device' => $disk_hash,
    };

    return (_make_recursive_hash($tmp_hash));
}

# This method permit to the user to delete an entity.
# The parameter is an hash containing parameters.
sub delete
{
    my ($OBJ, $param_hash) = @_;

    # Testing the read only mode..
    return(_make_error_hash(200, $error_hash->{200})) if ($OBJ->{read_only});

    # Testing if hash contains device key
    return(_make_error_hash(101, $error_hash->{101})) if (!exists($param_hash->{'device'}));

    # Looping on the hash
    foreach my $device (@{$param_hash->{'device'}})
    {
	# Testing if hash contains name key
	return(_make_error_hash(102, $error_hash->{102})) if (!exists($device->{'name'}));

	# This variable contains the device name
	my $device_name = $device->{'name'}[0];

	# If there is some partitions
	if (exists($device->{'partition'}))
	{
	    foreach my $partition (@{$device->{'partition'}})
	    {
		# Testing if partitions name are present
		return(_make_error_hash(103, $error_hash->{103})) if (!exists($partition->{'name'}));

		# Testing if partition belongs to the device
		return(_make_error_hash(201, $error_hash->{201})) if ($partition->{'name'}[0] !~ m/^($device_name).*/);

		# Deleting partition..
		my ($err, $msg) = Lib_Disk::delete_partition($partition->{'name'}[0]);
		# Testing for error..
		return(_make_error_hash(108, $error_hash->{108}.$msg)) if ($err);
	    }
	}
    }
    
    return(_make_error_hash(0, $error_hash->{0}));
}

# This method permit to the user to get all available method
# There is no parameter.
sub list_methods
{
    my $OBJ = shift;

    my $hash_return = {
	'methods' => [
		      'list_error_codes',
		      'get',
		      'info',
		      ],
	};
   
    push(@{$hash_return->{'methods'}}, "set") if(!$OBJ->{read_only});
    push(@{$hash_return->{'methods'}}, "delete") if(!$OBJ->{read_only});

    return($hash_return);
}


# This method permit to the user to get all error codes.
# There is no parameter.
sub list_error_codes
{
    my $hash_return = {
	'error_codes' => []
    };

    foreach my $key (sort keys(%{$error_hash}))
    {
	my $tmp_hash = {
	    'error' => [$key],
	    'msg' => [$error_hash->{$key}],
	};
	push(@{$hash_return->{'error_codes'}}, $tmp_hash);
    }

    return($hash_return);
}

### Private methods
sub _make_error_hash
{
    my ($error_code, $error_msg) = @_;

    my $hash_error = {
	'error_code' => [$error_code],
	'error_msg' => [$error_msg]
	};

    return($hash_error);
}

sub _make_recursive_hash
{
    my ($hash, $name_key) = @_;

    my $hash_return = {};
    foreach my $key (sort(keys(%{$hash})))
    {
	if (ref($hash->{$key}) eq "HASH")
	{
	    foreach my $hash_key (sort(keys(%{$hash->{$key}})))
	    {
		if ($hash_key =~ m/\/dev\//)
		{
		    push @{$hash_return->{$key}}, _make_recursive_hash($hash->{$key}->{$hash_key}, $hash_key);
		}
		else
		{
		    push @{$hash_return->{$key}}, _make_recursive_hash($hash->{$key});
		}
	    }
	}
	else
	{
	    push @{$hash_return->{$key}}, $hash->{$key};
	}
    }
    push @{$hash_return->{'name'}}, $name_key if ($name_key);

    return ($hash_return);
}

sub _unmake_hash
{
    my ($hash) = @_;

    my $hash_return = {};

    foreach my $tmp_hash (@{$hash->{'device'}})
    {
	my $sub_hash = _unmake_recursive_hash($tmp_hash);
	foreach my $sub_key (sort(keys(%{$sub_hash})))
	{
	    $hash_return->{$sub_key} = $sub_hash->{$sub_key};
	}
    }

    return ($hash_return);
}

sub _unmake_recursive_hash
{
    my ($hash) = @_;

    my $hash_return = {};

    my $dev = $hash->{'name'}[0];

    foreach my $key (sort(keys(%{$hash})))
    {
	if (ref($hash->{$key}[0]) eq 'HASH')
	{
	    foreach my $tmp_hash (@{$hash->{$key}})
	    {
		my $sub_hash = _unmake_recursive_hash($tmp_hash);
		foreach my $sub_key (sort(keys(%{$sub_hash})))
		{
		    $hash_return->{$dev}->{$key}->{$sub_key} = $sub_hash->{$sub_key};
		}
	    }
	}
	else
	{
	    $hash_return->{$dev}->{$key} = $hash->{$key}[0] if ($key ne 'name');
	}
    }

    return ($hash_return);
}

1;
